#!/usr/bin/python
"""Check-MK-Check for lmsensors-Values"""
# Example for output from agent
# ---------------------------------------------------------
#<<<lmsensors2>>>
#dell_smm_virtual_0_Processor_Fan 886 RPM
#dell_smm_virtual_0_Motherboard_Fan 905 RPM
#coretemp_isa_0000_Package_id_0 +36.0 C  high  +80.0 C crit  +98.0 C
#coretemp_isa_0000_Core_0 +35.0 C  high  +80.0 C crit  +98.0 C
#coretemp_isa_0000_Core_1 +36.0 C  high  +80.0 C crit  +98.0 C
#coretemp_isa_0000_Core_2 +36.0 C  high  +80.0 C crit  +98.0 C
#coretemp_isa_0000_Core_3 +33.0 C  high  +80.0 C crit  +98.0 C
#temp_radeon_pci_0100_temp1 +43.0 C  crit  +120.0 C hyst  +90.0 C
# ---------------------------------------------------------
#<<<lmsensors2>>>
#w83795adg_i2c_0_2f_in0 +0.10 V  min   +0.09 V max   +0.11 V
#w83795adg_i2c_0_2f_in1 +1.01 V  min   +0.60 V max   +1.49 V
#w83795adg_i2c_0_2f_Vbat +3.20 V  min   +2.70 V max   +3.63 V
#w83795adg_i2c_0_2f_fan2 4179 RPM  min   709 RPM
#w83795adg_i2c_0_2f_fan3 4272 RPM  min   709 RPM
#w83795adg_i2c_0_2f_fan4 4115 RPM  min   709 RPM
#w83795adg_i2c_0_2f_temp1 +56.5 C  high  +85.0 C hyst  +80.0 C crit  +100.0 C hyst  +95.0 C  sensor  thermal diode
#coretemp_isa_0000_Core_0 +36.0 C  high  +87.0 C crit  +97.0 C
#coretemp_isa_0000_Core_2 +33.0 C  high  +87.0 C crit  +97.0 C
#w83627hf_isa_0a00_in0 +4.08 V  min   +0.00 V max   +4.08 V
#w83627hf_isa_0a00_in1 +4.08 V  min   +0.00 V max   +4.08 V
#w83627hf_isa_0a00_in2 +4.08 V  min   +3.20 V max   +4.08 V
#w83627hf_isa_0a00_cpu0_vid +1.419 V
# ---------------------------------------------------------
#<<<lmsensors2>>>
#nct6779_isa_0290_Vcore 888.00 mV min   +0.00 V max   +1.74 V
#nct6779_isa_0290_in1 1.03 V  min   +0.00 V max   +0.00 V  ALARM
#nct6779_isa_0290_AVCC 3.39 V  min   +2.98 V max   +3.63 V
#nct6779_isa_0290_fan1 756 RPM  min     0 RPM
#temp_nct6779_isa_0290_SYSTIN +35.0 C  high   +0.0 C hyst   +0.0 C  ALARM  sensor  thermistor
#temp_nct6779_isa_0290_CPUTIN +31.0 C  high  +80.0 C hyst  +75.0 C  sensor  thermistor
#temp_nct6779_isa_0290_AUXTIN3 +6.0 C    sensor  thermal diode
#temp_nct6779_isa_0290_PECI_Agent_0 +28.5 C
#nct6779_isa_0290_intrusion0 ALARM
#nct6779_isa_0290_beep_enable disabled
#
# ---------------------------------------------------------

#from .agent_based_api.v1 import *
from .agent_based_api.v1 import (
        Metric,
        register,
        render,
        Result,
        Service,
        State,
        )


def discover_lmsensors2(section):
    """Discover lmsensors2"""
    for sensor in section:
        yield Service(item=sensor)

def parse_lmsensors2(string_table):
    """parse input"""
    def read_threshold(limitname, value, unit):
        if limitname not in ('ALARM', 'sensor'):
            value = float(value)
            if unit == "mV":
                value = value/1000

            if limitname == 'min':
                instance["crit_low"] = value
            elif limitname in ('max', 'crit', 'high'):
                if instance["crit_high"] is not None:
                    if value < instance["crit_high"]:
                        instance["warn_high"] = value
                    elif value > instance["crit_high"]:
                        instance["warn_high"] = instance["crit_high"]
                        instance["crit_high"] = value
                else:
                    instance["crit_high"] = value
            elif limitname == 'low':
                instance["crit_low"] = value


    parsed = {}
    for line in string_table:
        instance = parsed.setdefault(
            line[0], {
                "value": None,
                "unit": None,
                "crit_low": None,
                "warn_low": None,
                "warn_high": None,
                "crit_high": None,
                }
            )

        if len(line) > 2:
            unit = line[2]
            value = float(line[1])
            if unit == "mV":
                unit = "V"
                value = value/1000
            instance["value"] = value
            instance["unit"] = unit
        else:
            # beep_enabled and intrusion have only a status message
            instance["value"] = line[1]

        index = 5
        while len(line) > index:
            read_threshold(line[index -2], line[index -1], line[index])
            index = index + 3

    return parsed

def metric_name(unit, item):
    """select perfname by unit"""
    switcher = {
        'f':'temp',
        'F':'temp',
        'C':'temp',
        'c':'temp',
        'v':'voltage',
        'V':'voltage',
        'w':'power_usage',
        'W':'power_usage',
        'rpm':'fan',
        'RPM':'fan',
        }
    return switcher.get(unit, item)

def check_lmsensors2(item, section):
    """Check the values"""
    found = False
    return_val = State.OK
    return_message = "OK"
    for name in section:
        sensor = section[name]
        if sensor["value"] is not None and name == item:
            found = True
            break
    if not found:
        yield Result(
            state=State.UNKNOWN,
            summary="UNKNOWN - sensor status not found in agent output"
            )
    if sensor["unit"] is not None:
        perfcrit = read_perf_min_max(sensor["crit_low"], sensor["crit_high"])
        perfwarn = read_perf_min_max(sensor["warn_low"], sensor["warn_high"])
        yield Metric(
            metric_name(sensor["unit"], item),
            sensor['value'],
            levels=(perfwarn, perfcrit)
            )

    if sensor['value'] == "ALARM":
        yield Result(state=State.CRIT, summary="CRITICAL - Sensor value is ALARM")

    return_message = "OK - Sensor value is %s %s" % (sensor["value"], sensor["unit"])
    if sensor["crit_high"] is not None and sensor["value"] > sensor["crit_high"]:
        return_val = State.CRIT
        return_message = "CRITICAL - Sensor value %s %s, which is bigger than %s" % (
            sensor["value"],
            sensor["unit"],
            sensor["crit_high"]
            )
    if sensor["crit_low"] is not None and sensor["value"] < sensor["crit_low"]:
        return_val = State.CRIT
        return_message = "CRITICAL - Sensor value %s %s, which is smaller than %s" % (
            sensor["value"],
            sensor["unit"],
            sensor["crit_low"]
            )
    if sensor["warn_high"] is not None and sensor["value"] > sensor["warn_high"]:
        return_val = State.WARN
        return_message = "WARNING - Sensor value %s %s, which is bigger than %s" % (
            sensor["value"],
            sensor["unit"],
            sensor["warn_high"]
            )
    if sensor["warn_low"] is not None and sensor["value"] < sensor["warn_low"]:
        return_val = State.WARN
        return_message = "WARNING - Sensor value %s %s, which is smaller than %s" % (
            sensor["value"],
            sensor["unit"],
            sensor["warn_low"]
            )

    yield Result(state=return_val, summary=return_message)

def read_perf_min_max(smin, smax):
    """Lege die Schwellenwerte fuer die Performancedaten fest"""
    pcrit = None
    #if smax is not None and smin is not None:
    #    pcrit = '%s:%s' % (smin, smax)
    #elif smin is not None:
    if smin is not None:
        pcrit = smin
    elif smax is not None:
        pcrit = smax

    return pcrit

register.check_plugin(
        name="lmsensors2",
        service_name="Sensor %s",
        discovery_function=discover_lmsensors2,
        check_function=check_lmsensors2,
        )

register.agent_section(
        name="lmsensors2",
        parse_function=parse_lmsensors2,
        )
